#include <stdlib.h>
#include <time.h>

#include "place.h"
#include "trandom.h"
#include "writer.h"

void
t_terrain_place_objects (TTerrain *terrain,
                         gchar    *name,
                         gfloat    elevation_min,
                         gfloat    elevation_max,
                         gfloat    density,
                         gfloat    variance,
                         gfloat    scale_x,
                         gfloat    scale_y,
                         gfloat    scale_z,
                         gfloat    variance_x,
                         gfloat    variance_y,
                         gfloat    variance_z,
                         gboolean  vary_direction,
                         gboolean  symmetrical,
                         gint      seed)
{
  gfloat x, y;
  gfloat cell_width, cell_height;
  gfloat pov_size_x, pov_size_y, pov_size_z;

  TRandom *rnd;

  if (seed == -1)
    seed = time (NULL);

  rnd = t_random_new (seed);
  srand (seed);

  /* Determine terrain size in POV-Ray space */
  t_terrain_get_povray_size (terrain, &pov_size_x, &pov_size_y, &pov_size_z);

  /* calculate cell size */
  cell_width = 1.0 / (pov_size_x / scale_x * density);
  cell_height = 1.0 / (pov_size_z / scale_z * density); 

  /* loop through the squares on Pov and Hf offsets */
  for (y = cell_height * 0.5; y <= 1.0; y += cell_height)
    for (x = cell_width * 0.5; x <= 1.0; x += cell_width)
      {
        gfloat angle;
        gfloat position_x, position_z;
        gfloat size_x, size_y, size_z;

        /* calculate the object position in the scene */
        position_x =
          x + cell_width * variance * (t_random_rnd1 (rnd) - 0.5);
        position_z =
          y + cell_height * variance * (t_random_rnd1 (rnd) - 0.5);

        /* calculate the object scaling factors */
        if (symmetrical)
          {
            float t;

            t = t_random_rnd1 (rnd) - 0.5;
            size_x = scale_x * (1.0 + variance_x * t);
            size_y = scale_y * (1.0 + variance_y * t);
            size_z = scale_z * (1.0 + variance_z * t);
          }
        else
          {
            size_x =
              scale_x * (1.0 + variance_x * (t_random_rnd1 (rnd) * 2.0 - 1.0));
            size_y =
              scale_y * (1.0 + variance_y * (t_random_rnd1 (rnd) * 2.0 - 1.0));
            size_z =
              scale_z * (1.0 + variance_z * (t_random_rnd1 (rnd) * 2.0 - 1.0));
          }

        if (position_x >= 0.0 && position_x <= 1.0 &&
            position_z >= 0.0 && position_z <= 1.0)
          {
            gfloat height;
            gint   hf_x, hf_y, pos;

            hf_x = position_x * (terrain->width - 1);
            hf_y = position_z * (terrain->height - 1);
            pos = hf_y * terrain->width + hf_x;

            if (vary_direction)
              angle = rand () % 360;
            else
              angle = 0.0;

            height = terrain->heightfield[pos];
            if (height >= elevation_min && height <= elevation_max)
              if (terrain->selection == NULL ||
                  terrain->selection[pos] > 0.5)
                t_terrain_add_object (terrain,
                                      position_x, position_z,
                                      angle,
                                      size_x, size_y, size_z, name);
          }
      }

  t_random_free (rnd);
}

