/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
 *  - http://terraform.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __T_RGB_H__
#define __T_RGB_H__

#include <gtk/gtk.h>

typedef enum TSelectionType TSelectionType;

enum TSelectionType
{
  T_SELECTION_NONE,
  T_SELECTION_RECTANGLE,
  T_SELECTION_ELLIPSE
};

typedef struct TRgb TRgb;

struct TRgb
{
  GdkGC          *xor_gc;
  TSelectionType  selection_type;
  GdkRectangle    selection_bounds;
  GArray         *object_array;
  GtkWidget      *drawing_area;
  gint            width, height;
  guchar         *data;
  gint            dirty_x1, dirty_y1;
  gint            dirty_x2, dirty_y2;
  gint            sync_timeout;
  guchar          fore[3], back[3];
  gboolean        ignore_expose;

  guint           configure_id;
  guint           destroy_id;
  guint           expose_id;
  guint           realize_id;
  guint           unrealize_id;
};

TRgb      *t_rgb_new              (GtkWidget *drawing_area,
                                   gint       width,
                                   gint       height);
void       t_rgb_free             (TRgb      *rgb);
void       t_rgb_set_size         (TRgb      *rgb,
                                   gint       width,
                                   gint       height);
void       t_rgb_set_fore         (TRgb      *rgb,
                                   gint       red,
                                   gint       green,
                                   gint       blue);
void       t_rgb_set_back         (TRgb      *rgb,
                                   gint       red,
                                   gint       green,
                                   gint       blue);
void       t_rgb_invalidate       (TRgb      *rgb,
                                   gint       x1,
                                   gint       y1,
                                   gint       x2,
                                   gint       y2);
void       t_rgb_draw_raster      (TRgb      *rgb,
                                   gint       y,
                                   guchar    *data);
void       t_rgb_draw_pel         (TRgb      *rgb,
                                   gint       x,
                                   gint       y);
void       t_rgb_get_pel          (TRgb      *rgb,
                                   gint       x,
                                   gint       y,
                                   gint      *r,
                                   gint      *g,
                                   gint      *b);
void       t_rgb_draw_line        (TRgb      *rgb,
                                   gint       x1,
                                   gint       y1,
                                   gint       x2,
                                   gint       y2);
void       t_rgb_draw_line_add    (TRgb      *rgb,
                                   gint       x1,
                                   gint       y1,
                                   gint       x2,
                                   gint       y2);
void       t_rgb_draw_arrow       (TRgb      *rgb,
                                   gint       x1,
                                   gint       y1,
                                   gint       x2,
                                   gint       y2);
void       t_rgb_draw_rect        (TRgb      *rgb,
                                   gint       x1,
                                   gint       y1,
                                   gint       x2,
                                   gint       y2);
void       t_rgb_draw_poly        (TRgb      *rgb,
                                   gint       point_count,
                                   GdkPoint  *points);
void       t_rgb_render_static    (TRgb      *rgb);
void       t_rgb_clear            (TRgb      *rgb);
void       t_rgb_fill_triangle    (TRgb      *rgb,
                                   GdkPoint  *points);
void       t_rgb_fill_rect        (TRgb      *rgb,
                                   gint       x1,
                                   gint       y1,
                                   gint       x2,
                                   gint       y2);
TSelectionType t_rgb_get_selection(TRgb      *rgb,
                                   gfloat    *x1,
                                   gfloat    *y1,
                                   gfloat    *x2,
                                   gfloat    *y2);
void       t_rgb_set_selection    (TRgb           *rgb,
                                   TSelectionType  select_type,
                                   gint            x1,
                                   gint            y1,
                                   gint            x2,
                                   gint            y2);
gint       t_rgb_add_object       (TRgb      *rgb,
                                   gfloat     x,
                                   gfloat     y,
                                   gfloat     angle);
void       t_rgb_modify_object    (TRgb      *rgb,
                                   gint       item,
                                   gfloat     x,
                                   gfloat     y,
                                   gfloat     angle);
void       t_rgb_get_object       (TRgb      *rgb,
                                   gint       item,
                                   gfloat    *x,
                                   gfloat    *y,
                                   gfloat    *angle);
void       t_rgb_clear_objects    (TRgb      *rgb);
void       t_rgb_soft_sync        (TRgb      *sync);
void       t_rgb_hard_sync        (TRgb      *sync);
GtkWidget *t_rgb_get_drawing_area (TRgb      *rgb);
void       t_rgb_ignore_expose    (TRgb      *rgb,
                                   gboolean   ignore);

#endif /* __T_RGB_H__ */
